library IEEE;
library CRIS_LIBRARY;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use CRIS_LIBRARY.alupack.all;

-------------------------------------------------------------------------------
entity testbench is
end testbench;

-------------------------------------------------------------------------------
architecture test of testbench is

  type table_type1 is array (0 to 5) of signed (3 downto 0);
  type table_type2 is array (0 to 3) of std_logic_vector (1 downto 0);
  constant inputa : signed := "0000";
  constant inputb : signed := "0000";
  constant outc : table_type1 := ("0001", "0011", "0101", "0111", "1001","1011");
  constant outgen : table_type2 := ("00", "01", "10", "11");
  signal cbus : signed (3 downto 0);
  signal flag : std_logic;
  signal abus : signed (3 downto 0) := "0000";
  signal bbus : signed (3 downto 0) := "0000";
  signal clk : std_logic;
  signal reset : std_logic;
  signal sel : std_logic_vector (1 downto 0) := "00";

  component alu
    port (
      a, b : in stdbus;
      func : in std_logic_vector(1 downto 0);
      clk, reset : in std_logic;
      flag : out std_logic;
      c : out stdbus);
  end component;
  
  for alu_inst : alu use entity work.alu(rtl);

begin
  -- NOTE: simulation should be run for 2600 ns;
  alu_inst : alu port map (abus, bbus, sel, clk, reset, flag, cbus);

  clkp : process
  begin
    clk <= '1', '0' after 50 ns;
    wait for 100 ns;
  end process clkp;

  rset : process
  begin
    reset <= '1', '0' after 100 ns;
    wait for 1 ms;
  end process rset;

  testp : process
  begin
    wait for 100 ns; -- this is needed for reset to finish
    for j in 0 to 1 loop -- test for unsigned & signed add
      sel <= outgen(j);
      for i in 0 to 5 loop
        abus <= inputa + TO_SIGNED(i, 4);
        bbus <= inputb + TO_SIGNED(i+1, 4);
        wait for 51 ns;
        assert (cbus = outc(i))
        report "Result is not correct"
        severity warning;
        wait for 49 ns;
      end loop;
    end loop;

    for j in 2 to 3 loop -- test for unsigned & signed sub
      sel <= outgen(j);
      for i in 0 to 5 loop
        abus <= inputa + TO_SIGNED(i, 4);
        bbus <= inputb + TO_SIGNED(i+1, 4);
        wait for 51 ns;
        assert (cbus = "1111")
        report "Result is not correct"
        severity warning;
        wait for 49 ns;
      end loop;
    end loop;
    
    assert false
    report "Test Complete"
    severity error;
    
  end process testp;

end test;
-------------------------------------------------------------------------------
